@@ -48,7 +48,7 @@ class Projection(collections.abc.Mapping):
     """
 
     def __init__(self, keys, space):
-        self._keys = tuple(keys)
+        self._keys = set(keys)
         self._space = space
 
     def __getitem__(self, key):
@@ -56,11 +56,14 @@ class Projection(collections.abc.Mapping):
             raise KeyError(key)
         return self._space[key]
 
+    def _keys_resolved(self):
+        return self._keys.intersection(self._space)
+
     def __iter__(self):
-        return iter(set(self._keys).intersection(self._space))
+        return iter(self._keys_resolved())
 
     def __len__(self):
-        return len(tuple(iter(self)))
+        return len(self._keys_resolved())
 
 
 class DictFilter(collections.abc.Mapping):
